import os
from argparse import Namespace
from pathlib import Path

from dj_settings import SettingsParser

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv: str = options.venv
        self.cd: bool = options.cd

    def run(self):
        venv_path = self.base_dir.joinpath(self.venv)
        if not venv_path.exists():
            raise RuntimeError(f"Venv {self.venv} doesn't exist, aborting...")

        if os.getenv("VIRTUAL_ENV"):
            print("dvenv")

        project = venv_path.joinpath(".project")
        if project.exists():
            if self.cd:
                with open(project) as file:
                    print(f"cd {file.read().strip()}")
            environment = venv_path.joinpath(".environment")
            if environment.exists():
                new_environment = {}
                with open(environment) as file:
                    for line in file:
                        new_environment |= SettingsParser(line.strip()).data
                environment_string = " ".join(
                    f"{key}={value}" for key, value in new_environment.items()
                )
                print(f"invenv {environment_string}")

        activate = venv_path.joinpath("bin/activate")
        if activate.exists():
            print(f". {activate}")
        else:
            print(f"export VIRTUAL_ENV=dummy_{self.venv}")



import shutil
from argparse import Namespace
from pathlib import Path
from typing import List

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv_to_remove: List[str] = options.venvs_to_remove

    def run(self):
        for venv in self.venv_to_remove:
            venv_path = self.base_dir.joinpath(venv)
            if not venv_path.exists():
                print(f"{venv} does not exist, skipping...")
                continue
            try:
                shutil.rmtree(venv_path)
            except (PermissionError, NotADirectoryError):
                print(f"Cannot delete {venv}, skipping...")
                continue

            print(f"{venv} removed successfully!")



import os
from argparse import Namespace
from typing import List

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)
        self.env_vars: List[str] = options.env_vars
        self.files = options.files

    @staticmethod
    def parse_env_var(line: str) -> List[str]:
        return line.split("=", maxsplit=1)

    def run(self):
        new_vars = {}
        for file in self.files:
            with open(file) as f:
                for line in f:
                    line = line.strip()
                    if line and not line.startswith("#"):
                        key, value = self.parse_env_var(line)
                        new_vars[key] = value
        for env_var in self.env_vars:
            key, value = self.parse_env_var(env_var)
            new_vars[key] = value

        if new_vars:
            if self._prefix in os.environ:
                raise RuntimeError("Already in a venv, aborting...")
            print(f"export {self._prefix}=true")
        for key, value in new_vars.items():
            if key in os.environ:
                print(f"export {self._prefix}_{key}={os.getenv(key, '')}")
            else:
                print(f"export {self._prefix}_unset_{key}=")
            print(f"export {key}={value}")



import argparse
import os
import sys
from pathlib import Path
from types import ModuleType

from pvenv import __version__, subcommands

sys.tracebacklimit = 0


def get_default_base() -> Path:
    default = f"{os.path.expanduser('~')}/.local/share/virtualenvs"
    return Path(os.getenv("PVENV_BASE", default))


def parse_args():
    parser = argparse.ArgumentParser(
        prog="venv",
        description="A utility to manage virtual environments",
        epilog="You can find the full documentation at: https://p-venv.readthedocs.io/en/stable/",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"%(prog)s {__version__}",
        help="Print the version and exit",
    )
    parser.add_argument("base_dir", action="store_const", const=get_default_base())
    subparsers = parser.add_subparsers(dest="command")

    invenv_parser = subparsers.add_parser(
        "in", help="Export new variables for the venv"
    )
    invenv_parser.add_argument("env_vars", nargs="*")
    invenv_parser.add_argument("-f", "--files", action="append", default=[])
    subparsers.add_parser("out", help="List venvs")

    avenv_parser = subparsers.add_parser("activate", help="Activate a venv")
    avenv_parser.add_argument("venv")
    avenv_parser.add_argument("-n", "--no-cd", dest="cd", action="store_false")
    subparsers.add_parser("deactivate", help="Deactivate a venv")
    subparsers.add_parser("list", help="List venvs")
    mkvenv_parser = subparsers.add_parser("make", help="Make a new venv")
    mkvenv_parser.add_argument("venv", nargs="?", default=Path().absolute().name)
    mkvenv_parser.add_argument("-e", "--environments", action="append", default=[])
    mkvenv_parser.add_argument("-P", "--project", default=Path().absolute().as_posix())
    mkvenv_parser.add_argument("-p", "--python", default="current")

    rmvenv_parser = subparsers.add_parser("rm", help="Remove virtualenvs")
    rmvenv_parser.add_argument("venvs_to_remove", nargs="*")
    return parser.parse_args()


def main() -> None:
    args = parse_args()
    module: ModuleType
    if args.command == "in":
        module = subcommands.invenv
    elif args.command == "out":
        module = subcommands.outvenv
    elif args.command == "activate":
        module = subcommands.avenv
    elif args.command == "deactivate":
        module = subcommands.dvenv
    elif args.command == "list":
        module = subcommands.lsvenv
    elif args.command == "make":
        module = subcommands.mkvenv
    elif args.command == "rm":
        module = subcommands.rmvenv
    else:
        print("Add the following line to your shell rc:")
        print(f". {Path(__file__).parent.joinpath('scripts/pvenv.sh').absolute()}")
        return
    module.Command(args).run()



import os
from argparse import Namespace

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)

    def run(self):
        print("outvenv")
        print(f"declare -f deactivate > {os.devnull} && deactivate")
        if os.getenv("VIRTUAL_ENV"):
            print("unset VIRTUAL_ENV")



import os
from argparse import Namespace

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)

    def run(self):
        if self._prefix not in os.environ:
            return

        print(f"unset {self._prefix}")
        prefix_size = len(self._prefix) + 1
        unset_prefix_size = len(self._prefix) + len("_unset") + 1
        for key, value in os.environ.items():
            if key.startswith(f"{self._prefix}_"):
                if key.startswith(f"{self._prefix}_unset_"):
                    print(f"unset {key[unset_prefix_size:]}")
                else:
                    print(f"export {key[prefix_size:]}={value}")
                print(f"unset {key}")



import os
from argparse import Namespace
from pathlib import Path
from typing import List

from pvenv.subcommands.base import BaseCommand

DEV_NULL = Path(os.devnull)


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv: str = options.venv
        self.python: str = options.python
        self.project: Path = Path(options.project).absolute()
        self.environments: List[Path] = self._get_environments()

    def _get_project(self) -> Path:
        project = self._options.project
        return Path(project).absolute() if project else DEV_NULL

    def _get_environments(self) -> List[Path]:
        environments = self._options.environments
        if self.project == DEV_NULL and environments:
            raise RuntimeError("Cannot set environ in independent")
        return [self.project.joinpath(environ) for environ in environments]

    def run(self):
        venv_path = self.base_dir.joinpath(self.venv)
        if venv_path.exists():
            raise RuntimeError(f"Venv {self.venv} already exists, aborting...")

        if self.python == "current":
            print(f"python -m venv {venv_path}")
        elif self.python:
            print(f"pyenv shell {self.python}")
            print(f"python -m venv {venv_path}")
        else:
            print(f"mkdir -p {venv_path}")

        if self.project != DEV_NULL:
            print(f"echo {self.project} > {venv_path}/.project")

        for i, environment in enumerate(self.environments):
            if i == 0:
                print(f": > {venv_path}/.environment")
            print(f"echo {environment} >> {venv_path}/.environment")

        print(f"avenv {self.venv}")

        if self.python:
            print("pip install --upgrade pip wheel")



from argparse import Namespace


class BaseCommand:
    def __init__(self, options: Namespace):
        self._prefix = "_pvenv_env"
        self._options = options

    def run(self):
        raise NotImplementedError(f"{self.__class__.__name__} must implement run")



from argparse import Namespace
from pathlib import Path

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace):
        super().__init__(options)
        self.base_dir: Path = options.base_dir

    def run(self):
        for directory in sorted(self.base_dir.glob("*")):
            if directory.is_dir():
                print(directory.name)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pvenv', 'pvenv.subcommands']

package_data = \
{'': ['*'], 'pvenv': ['scripts/*']}

install_requires = \
['dj_settings>=4.2.0,<5.0.0']

entry_points = \
{'console_scripts': ['pvenv = pvenv.main:main']}

setup_kwargs = {
    'name': 'pvenv',
    'version': '0.6.0',
    'description': 'Easy python virtual environment management',
    'long_description': '=================================================\npvenv: Easy python virtual environment management\n=================================================\n\n.. image:: https://github.com/spapanik/pvenv/actions/workflows/build.yml/badge.svg\n  :alt: Build\n  :target: https://github.com/spapanik/pvenv/actions/workflows/build.yml\n.. image:: https://img.shields.io/lgtm/alerts/g/spapanik/pvenv.svg\n  :alt: Total alerts\n  :target: https://lgtm.com/projects/g/spapanik/pvenv/alerts/\n.. image:: https://img.shields.io/github/license/spapanik/pvenv\n  :alt: License\n  :target: https://github.com/spapanik/pvenv/blob/main/LICENSE.txt\n.. image:: https://img.shields.io/pypi/v/pvenv\n  :alt: PyPI\n  :target: https://pypi.org/project/pvenv\n.. image:: https://pepy.tech/badge/pvenv\n  :alt: Downloads\n  :target: https://pepy.tech/project/pvenv\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n  :alt: Code style\n  :target: https://github.com/psf/black\n\n\nIn a nutshell\n-------------\n\nInstallation\n^^^^^^^^^^^^\n\nThe easiest way is to use pipx to install ``pvenv``.\n\n.. code:: console\n\n   $ pipx install pvenv\n\nPlease make sure that the correct directory is added to your path. This\ndepends on the OS.\n\nRun ``pvenv`` to get the path to be sourced in the shell rc:\n\n.. code:: console\n\n   $ pvenv\n\nUsage\n^^^^^\n\nYou can use the following commands:\n\n* `venv activate` (aliased to `avenv`) to activate a virtual environment\n* `venv deactivate` (aliased to `dvenv`) to deactivate a virtual environment\n* `venv list` (aliased to `lsvenv`) to list all the virtual environments\n* `venv make` (aliased to `mkvenv`) to create a new virtual environment\n* `venv rm` (aliased to `rmvenv`) to remove a virtual environment\n\n\nLinks\n-----\n\n- `Documentation`_\n- `Changelog`_\n\n\n.. _Changelog: https://github.com/spapanik/pvenv/blob/main/CHANGELOG.rst\n.. _Documentation: https://p-venv.readthedocs.io/en/stable/\n',
    'author': 'Stephanos Kuma',
    'author_email': 'stephanos@kuma.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/spapanik/pvenv',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)
